program define bhps_ukhls
/*damien.vdd@gmail.com*/
/*version 0.12 beta
Changes in 0.12 beta (13mar2014):
- Changed the merging/joinby process of compiling from keeping masterfile's missing values to updating them with the filelist
Changes in 0.11 beta:
- Added an error message if 'too many macros' bugs the vallabelcheck option
- Added a message each time a new variable is processed during the vallabcheck option execution
Initial version: 0.10 beta (07feb2014)
*/
/*This program is tested only for the files accessible from:
http://discover.ukdataservice.ac.uk/series/?sn=200005
with serial number "5151" ("British Household Panel Survey: Waves 1-18, 1991-2009")
and from:
http://discover.ukdataservice.ac.uk/series/?sn=2000053
with serial number "6614" ("Understanding Society: Waves 1-3, 2009-2012")
*/
/*It should be compatible with newcoming waves of Understanding Society though (type -help bhps_ukhls- and see section "compatibility issues")*/
/*Cross-wave files (filenames with the prefix "x") have not been tested yet*/
version 12
syntax [anything] using/ ,[BHPSWaves(numlist integer min=1 max=18 sort) UKHLSWaves(numlist integer min=1 max=26 sort) BHPSDir(string) UKHLSDir(string) Merge(string) VALlabcheck VARlabcheck Compact Justcheck Debugmode]

*Preamble:
if "`debugmode'"==""{
	local quimode quietly
	global noisily
}
else{
	local quimode noisily
	global noisily noisily
}

`quimode'{

local preserveisok 0
if "`justcheck'"!="" & _N>0{
	tempfile preserve
	local preserveisok 0
	save `preserve',replace
	local preserveisok 1
}
clear


if "`justcheck'"=="justcheck" & "`vallabcheck'"=="" & "`varlabcheck'"==""{
	local vallabcheck vallabcheck
	local varlabcheck varlabcheck
	n di "Note: The justcheck option has forced the activation of both options varlabcheck and vallabcheck because at least one of them has to be specified."
}

if "`bhpswaves'"=="" & "`ukhlswaves'"==""{
	local allfiles: dir "`ukhlsdir'" files "?_indresp*.dta"
	local nbfiles: word count `allfiles'
	if `nbfiles'>0{
		local lastfile: word `nbfiles' of `allfiles'
		local lastwave=index("abcdefghijklmnopqrstuvwxyz",substr("`lastfile'",1,1))
	}
	numlist "1/18"
	local bhpswaves: display r(numlist)
	if `nbfiles'>0{
		numlist "1/`lastwave'"
		local ukhlswaves: display r(numlist)
	}
	n di "Note: Without defining options bhpswaves and ukhlswaves, the program assumed you wanted all waves of both BHPS and UKHLS. Make sure this is what you want because the computation may become cumbersome."
}

global uid hid hidp pno ficode fiseq opno scpno cheseq marno lcsno lncno lacno leshno ljseq apno childpno newchno spellno absparno cohabno adoptno childno issueno callno
global filelist "`using' `merge'"
global filelist: list uniq global(filelist)
global varlist "$uid pid pidp `anything'"
global varlist: list uniq global(varlist)
global anything: list uniq local(anything)
local bhpswaves: list uniq local(bhpswaves)
local ukhlswaves: list uniq local(ukhlswaves)

global wavelist09
if "`bhpswaves'"!="" global wavelist09 `bhpswaves'
if "`ukhlswaves'"!=""{
	foreach nb of numlist `ukhlswaves'{
		local nb18: display `nb'+18
		global wavelist09 $wavelist09 `nb18'
	}
}


*Aggregate variables, waves and files
if "`justcheck'"==""{
tempfile compile master slave
local alluidused
foreach file of global filelist{
	clear
	if substr("`file'",1,1)=="x"{
		n di "This program does not implement cross-wave files (data files starting with 'x', such as `file').  The program is terminated without success."
		error 101
	}
	local firstwave 1
	foreach wave of global wavelist09{
		if `wave'<=18{
			local waveaz=substr("abcdefghijklmnopqrstuvwxyz",`wave',1)
			if "`bhpsdir'"=="" capture $noisily use `waveaz'`file',clear
			else capture $noisily use "`bhpsdir'/`waveaz'`file'",clear
			if _rc==601 continue
			capture $noisily if "`waveaz'"=="p" ren pid ppid
			renpfix "`waveaz'"
		}
		if `wave'>=19{
			local waveaz=substr("abcdefghijklmnopqrstuvwxyz",`wave'-18,1)
			if "`ukhlsdir'"=="" capture $noisily use `waveaz'_`file',clear
			else capture $noisily use "`ukhlsdir'/`waveaz'_`file'",clear
			if _rc==601 continue
			renpfix "`waveaz'_"
		}
		if "`anything'"!=""{
			local loopvarlist
			foreach var of global varlist{
				capture $noisily unab x: `var'
				if _rc==0 local loopvarlist `loopvarlist' `x'
			}
			capture $noisily keep `loopvarlist'
		}
		gen byte wave=`wave'
		if `firstwave'==0 append using `compile'
		save `compile',replace
		local firstwave 0
	}
	if c(N)==0{
		n di "This file (`file') seems not to exist in the specified directory. The program is terminated without success."
		error 601
	}
	la var wave "Wave ID"
	if "`file'"=="`using'"{
		ds
		local A `r(varlist)'
		local masterid: list local(A) & global(uid)
		save `master',replace
		continue
	}
	ds
	local A `r(varlist)'
	local slaveid: list local(A) & global(uid)
	local masterslaveid: list local(masterid) & local(slaveid)
	local alluidused `alluidused' `masterslaveid'
	capture $noisily isid `masterslaveid' wave, missok
	if _rc==459 local slave1m m
	else local slave1m 1
	save `slave',replace
	
	use `master',clear
	ds
	local A `r(varlist)'
	local masterid: list local(A) & global(uid)
	capture $noisily isid `masterslaveid' wave, missok
	if _rc==459 local master1m m
	else local master1m 1
	if "`master1m'"=="m" & "`slave1m'"=="m"{
		joinby `masterslaveid' wave using `slave', unmatched(master) update
		drop _merge
		n di "Note: At least two of the data files you specified do not have a unique identifying variable in common, and hence cannot be merged together. Consequently, pairwise combinations have been done to combine these data files (cartesian product)."
		}
	else merge `master1m':`slave1m' `masterslaveid' wave using `slave', keep(1 3 4) nogenerate update
	save `master',replace
	
} /*END OF if "`justcheck'"==""*/
if "`masterslaveid'"!=""{
	ds
	local A `r(varlist)'
	local otherid: list global(uid) - local(alluidused)
	local A: list local(A) - local(otherid)
	capture $noisily keep `A'
}
compress
save `master',replace
}

if "`vallabcheck'"=="" & "`varlabcheck'"=="" exit

*Check labelling continuity:
tempfile export export_vallab waveloopdta fileloopdta valuelabelsdo
local firstfile 1
foreach file of global filelist{
	clear
	if substr("`file'",1,1)=="x"{
		n di "This program does not implement cross-wave files (data files starting with 'x', such as `file').  The program is terminated without success."
		error 101
	}
	local firstwave 1
	foreach wave of global wavelist09{
		if `wave'<=18{
			local waveaz=substr("abcdefghijklmnopqrstuvwxyz",`wave',1)
			if "`bhpsdir'"=="" capture $noisily use `waveaz'`file',clear
			else capture $noisily use "`bhpsdir'/`waveaz'`file'",clear
			if _rc==601 continue
			capture $noisily if "`waveaz'"=="p" ren pid ppid
			renpfix "`waveaz'"
		}
		if `wave'>=19{
			local waveaz=substr("abcdefghijklmnopqrstuvwxyz",`wave'-18,1)
			if "`ukhlsdir'"=="" capture $noisily use `waveaz'_`file',clear
			else capture $noisily use "`ukhlsdir'/`waveaz'_`file'",clear
			if _rc==601 continue
			renpfix "`waveaz'_"
		}
		if "`anything'"!=""{
			local loopvarlist
			foreach var of global anything{
				capture $noisily unab x: `var'
				if _rc==0 local loopvarlist `loopvarlist' `x'
			}
			if "`loopvarlist'"!="" keep `loopvarlist'
			else{
				clear
				continue
			}
		}
		desc,replace clear
		levelsof vallab,clean local(vallabs)
		foreach vallab of local vallabs{
			capture $noisily label copy `vallab' `vallab'_`file'
			if _rc==111 replace vallab="" if vallab=="`vallab'"
			replace vallab="`vallab'_`file'" if vallab=="`vallab'"
		}
		levelsof vallab,clean local(vallabs)
		if "`vallabs'"!=""{
			label save `vallabs' using `valuelabelsdo',replace
			label drop _all
			run `valuelabelsdo'
		}
		else label drop _all
		gen byte wave=`wave'
		if `firstwave'==0 append using `waveloopdta'
		save `waveloopdta',replace orphans
		local firstwave 0
	}
	if c(N)==0{
		n di "This file (`file') does not exist in the specified directory, or does not contain variable labels for any of the specified variables. The program is terminated without success."
		error 601
	}
	gen file="`file'"
	if `firstfile'==0 append using `fileloopdta'
	save `fileloopdta',replace orphans
	local firstfile 0
}
la var wave "Wave ID"
la var name "Name of the variable"
la var file "File containing the variable"
egen name_file = concat(name file), punct(_)
la var name_file "Name and file of the variable"
save `export',replace orphans
capture $noisily erase bhpsukhls_labcheck.xls

*Generate excel spreadsheet for varlab:
if "`varlabcheck'"!=""{
	use `export',clear
	keep varlab wave name_file name file
	tab wave
	local nbvars=r(N)/r(r)
	if `nbvars'<=255 & "`compact'"==""{
		drop name file
		reshape wide varlab,i(wave) j(name_file) string
		order varlab*,alphabetic
		order wave,first
		renpfix varlab
	}
	else{
		drop name_file
		reshape wide varlab,i(name file) j(wave)
		renpfix varlab wave
		order *,sequential
		order name,first
		sort name file
		ren name variable
	}
	capture $noisily export excel using "bhpsukhls_labcheck", sheet("varlab") sheetreplace firstrow(variables)
	if _rc!=0{
		outsheet using "bhpsukhls_varlab.csv", comma replace
		n di "Note: Table of variable labels could not have been exported in excel format. The file bhpsukhls_varlab.csv has been generated instead."
	}
	else n di "Note: A codebook of variable labels has been generated in excel format (bhpsukhls_labcheck.xls) with the sheet name 'varlab', and placed into your working directory."
}

*Generate excel spreadsheet for vallab:
if "`vallabcheck'"!=""{
	use `export',clear
	capture uselabel,clear
	if _rc!=0{
		if _rc==920 n di "Varlist is too big, you need to reduce it or to increase Stata's abilities using, for instance 'set maxvar 32767'."
		if "`justcheck'"=="" use `master',clear
		else if "`justcheck'"!="" & `preserveisok'==1 use `preserve',clear
		else clear
		macro drop uid filelist varlist wavelist09 anything noisily
		error _rc
	}
	if c(N)==0{
		n di "Note: There is no value labels in the specified files for the specified variables. The option vallabcheck is therefore not executed."
		if "`justcheck'"=="" use `master',clear
		else if "`justcheck'"!="" & `preserveisok'==1 use `preserve',clear
		else clear
		macro drop uid filelist varlist wavelist09 anything noisily
		exit
	}
	drop trunc
	rename lname vallab
	merge m:m vallab using `export', keepusing(name file wave) keep(3) nogenerate
	drop vallab
	save `export_vallab',replace
	levelsof name,local(names)
	local nbvars: word count `names'
	if "`compact'"!="" local nbvars 33
	local firstloop 1
	foreach name of local names{
		use `export_vallab',clear
		keep if name=="`name'"
		reshape wide label,i(value file name) j(wave)
		if `nbvars'<=32 drop name
		renpfix label wave
		order *,sequential
		if `nbvars'>32 order name,first
		la var value "Value of the category"
		sort file value
		if `nbvars'>32{
			local varleft `ferest()'
			local nbvarleft: list sizeof local(varleft)
			n di "Processing labels of variable `name'... (`nbvarleft' variables left)"
			if `firstloop'==1{
				tempfile valexcel
				save `valexcel',replace
				local firstloop 0
			}
			else{
				append using `valexcel'
				save `valexcel',replace
			}
				
		}
		else{
			capture $noisily export excel using "bhpsukhls_labcheck",sheet("vallab_`name'") sheetreplace firstrow(variables)
			if _rc!=0{
				outsheet using "bhpsukhls_vallab_`name'.csv", comma replace
				n di "Note: Table of value labels of variable `name' could not have been exported in excel format. The file bhpsukhls_vallab_`name'.csv has been generated instead."
			}
			else n di "Note: A codebook of value labels for the variable `name' has been generated in excel format (bhpsukhls_labcheck.xls) with the sheet name 'vallab_`name'', and placed into your working directory."
		}
	}
	if `nbvars'>32{
		order name file value wave*,sequential
		ren name variable
		la var variable "Name of the variable"
		capture $noisily export excel using "bhpsukhls_labcheck",sheet("vallab") sheetreplace firstrow(variables)
		if _rc!=0{
			outsheet using "bhpsukhls_vallab.csv", comma replace
			n di "Note: Table of value labels could not have been exported in excel format. The file bhpsukhls_vallab.csv has been generated instead."
		}
		else n di "Note: A codebook of value labels has been generated in excel format (bhpsukhls_labcheck.xls) with the sheet name 'vallab', and placed into your working directory."
	}
}

if "`justcheck'"=="" use `master',clear
else if "`justcheck'"!="" & `preserveisok'==1 use `preserve',clear
else clear
macro drop uid filelist varlist wavelist09 anything noisily

} /*END OF `quimode'*/
end
